#ifndef DESKTOP_CLIENT_PROTOCOL_H
#define DESKTOP_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-util.h"

struct wl_client;
struct wl_resource;

struct desktop_shell;
struct screensaver;

extern const struct wl_interface desktop_shell_interface;
extern const struct wl_interface screensaver_interface;

struct desktop_shell_listener {
	/**
	 * configure - (none)
	 * @time: (none)
	 * @edges: (none)
	 * @surface: (none)
	 * @width: (none)
	 * @height: (none)

	 */
	void (*configure)(void *data,
			  struct desktop_shell *desktop_shell,
			  uint32_t time,
			  uint32_t edges,
			  struct wl_shell_surface *surface,
			  int32_t width,
			  int32_t height);
	/**
	 * prepare_lock_surface - (none)

	 */
	void (*prepare_lock_surface)(void *data,
				     struct desktop_shell *desktop_shell);
};

static inline int
desktop_shell_add_listener(struct desktop_shell *desktop_shell,
			   const struct desktop_shell_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) desktop_shell,
				     (void (**)(void)) listener, data);
}

#define DESKTOP_SHELL_SET_BACKGROUND	0
#define DESKTOP_SHELL_SET_PANEL	1
#define DESKTOP_SHELL_SET_LOCK_SURFACE	2
#define DESKTOP_SHELL_UNLOCK	3

static inline void
desktop_shell_set_user_data(struct desktop_shell *desktop_shell, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) desktop_shell, user_data);
}

static inline void *
desktop_shell_get_user_data(struct desktop_shell *desktop_shell)
{
	return wl_proxy_get_user_data((struct wl_proxy *) desktop_shell);
}

static inline void
desktop_shell_destroy(struct desktop_shell *desktop_shell)
{
	wl_proxy_destroy((struct wl_proxy *) desktop_shell);
}

static inline void
desktop_shell_set_background(struct desktop_shell *desktop_shell, struct wl_output *output, struct wl_shell_surface *surface)
{
	wl_proxy_marshal((struct wl_proxy *) desktop_shell,
			 DESKTOP_SHELL_SET_BACKGROUND, output, surface);
}

static inline void
desktop_shell_set_panel(struct desktop_shell *desktop_shell, struct wl_output *output, struct wl_shell_surface *surface)
{
	wl_proxy_marshal((struct wl_proxy *) desktop_shell,
			 DESKTOP_SHELL_SET_PANEL, output, surface);
}

static inline void
desktop_shell_set_lock_surface(struct desktop_shell *desktop_shell, struct wl_shell_surface *surface)
{
	wl_proxy_marshal((struct wl_proxy *) desktop_shell,
			 DESKTOP_SHELL_SET_LOCK_SURFACE, surface);
}

static inline void
desktop_shell_unlock(struct desktop_shell *desktop_shell)
{
	wl_proxy_marshal((struct wl_proxy *) desktop_shell,
			 DESKTOP_SHELL_UNLOCK);
}

#define SCREENSAVER_SET_SURFACE	0

static inline void
screensaver_set_user_data(struct screensaver *screensaver, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) screensaver, user_data);
}

static inline void *
screensaver_get_user_data(struct screensaver *screensaver)
{
	return wl_proxy_get_user_data((struct wl_proxy *) screensaver);
}

static inline void
screensaver_destroy(struct screensaver *screensaver)
{
	wl_proxy_destroy((struct wl_proxy *) screensaver);
}

static inline void
screensaver_set_surface(struct screensaver *screensaver, struct wl_shell_surface *surface, struct wl_output *output)
{
	wl_proxy_marshal((struct wl_proxy *) screensaver,
			 SCREENSAVER_SET_SURFACE, surface, output);
}

#ifdef  __cplusplus
}
#endif

#endif
